From: Rusty Russell <rusty@rustcorp.com.au>
To: He Kuang <hekuang@huawei.com>
Cc: wangnan0@huawei.com, linux-kernel@vger.kernel.org,
"Peter Zijlstra" <peterz@infradead.org>
Subject: Re: [PATCH] module: Fix missing to hold module_mutex lock in module_kallsyms_lookup_name
Date: Tue, 28 Jul 2015 10:01:35 +0930 [thread overview]
Message-ID: <878ua1p1g8.fsf@rustcorp.com.au> (raw)
In-Reply-To: <1437991286-101906-1-git-send-email-hekuang@huawei.com>
He Kuang <hekuang@huawei.com> writes:
> Function find_module_all() searches for module by name and must be
> called with module_mutex. module_kallsyms_lookup_name() calls it without
> this mutex which emits a warning message (CONFIG_LOCKDEP=y) by failed
> assertion for testing this module_mutex lock, as following:
>
> [ 202.877152] ------------[ cut here ]------------
> [ 202.881070] WARNING: CPU: 0 PID: 1010 at kernel/module.c:281
> module_assert_mutex+0x35/0x40()
> [ 202.885446] Modules linked in: test_bpf
> [ 202.886997] CPU: 0 PID: 1010 Comm: perf Tainted: G W 4.2.0-rc3+ #5
> ...
>
> This patch wraps this call with mutex_{lock,unlock} and fix the bug.
Hi He!
Thanks for this report! This warning is overzealous; preempt
disabling should be enough to read the list.
Unfortunately, as you can see from the comment, taking a lock is a bad
idea here: it's called in the oops path (we don't want to risk
deadlock).
Peter?
Cheers,
Rusty.
module: weaken locking assertion for oops path.
We don't actually hold the module_mutex when calling find_module_all
from module_kallsyms_lookup_name: that's because it's used by the oops
code and we don't want to deadlock.
However, access to the list read-only is safe if preempt is disabled,
so we can weaken the assertion. Keep a strong version for external
callers though.
Fixes: 0be964be0d45 ("module: Sanitize RCU usage and locking")
Reported-by: He Kuang <hekuang@huawei.com>
Signed-off-by: Rusty Russell <rusty@rustcorp.com.au>
diff --git a/kernel/module.c b/kernel/module.c
index 4d2b82e610e2..b86b7bf1be38 100644
--- a/kernel/module.c
+++ b/kernel/module.c
@@ -602,13 +602,16 @@ const struct kernel_symbol *find_symbol(const char *name,
}
EXPORT_SYMBOL_GPL(find_symbol);
/* Search for module by name: must hold module_mutex. */
static struct module *find_module_all(const char *name, size_t len,
bool even_unformed)
{
struct module *mod;
- module_assert_mutex();
+ module_assert_mutex_or_preempt();
list_for_each_entry(mod, &modules, list) {
if (!even_unformed && mod->state == MODULE_STATE_UNFORMED)
@@ -621,6 +624,7 @@ static struct module *find_module_all(const char *name, size_t len,
struct module *find_module(const char *name)
{
+ module_assert_mutex();
return find_module_all(name, strlen(name), false);
}
EXPORT_SYMBOL_GPL(find_module);
next prev parent reply other threads:[~2015-07-28 0:31 UTC|newest]
Thread overview: 3+ messages / expand[flat|nested] mbox.gz Atom feed top
2015-07-27 10:01 [PATCH] module: Fix missing to hold module_mutex lock in module_kallsyms_lookup_name He Kuang
2015-07-28 0:31 ` Rusty Russell [this message]
2015-07-28 12:52 ` Peter Zijlstra
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=878ua1p1g8.fsf@rustcorp.com.au \
--to=rusty@rustcorp.com.au \
--cc=hekuang@huawei.com \
--cc=linux-kernel@vger.kernel.org \
--cc=peterz@infradead.org \
--cc=wangnan0@huawei.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -602,13 +602,16 @@ const struct kernel_symbol *find_symbol(const char *name,
}
EXPORT_SYMBOL_GPL(find_symbol);
-/* Search for module by name: must hold module_mutex. */
+/*
+ * Search for module by name: must hold module_mutex (or preempt disabled
+ * for read-only access).
+ */
static struct module *find_module_all(const char *name, size_t len,
bool even_unformed)
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox